@@ -81,7 +81,7 @@ information is kept on a per-level basis.
 
 (3) Sometimes the widget instance tree is structured such that you get the same
 class name repeated on the fully qualified widget name.  This can result in the
-same database level occuring multiple times on the search list.  The code below
+same database level occurring multiple times on the search list.  The code below
 only checks to see if you get two identical search lists in a row, rather than
 look back through all database levels, but in practice this removes all
 duplicates I've ever observed.
@@ -1047,7 +1047,7 @@ XrmQPutStringResource(
 
     if (!*pdb) *pdb = NewDatabase();
     value.addr = (XPointer) str;
-    value.size = strlen(str)+1;
+    value.size = (unsigned) strlen(str) + 1;
     _XLockMutex(&(*pdb)->linfo);
     PutEntry(*pdb, bindings, quarks, XrmQString, &value);
     _XUnlockMutex(&(*pdb)->linfo);
@@ -1307,7 +1307,7 @@ static void GetDatabase(
 	    char oldc;
 
 	    /*
-	     * A parsing error has occured, toss everything on the line
+	     * A parsing error has occurred, toss everything on the line
 	     * a new_line can still be escaped with a '\'.
 	     */
 
@@ -1477,7 +1477,7 @@ static void GetDatabase(
 	    /*
 	     * It is important to make sure that there is room for at least
 	     * four more characters in the buffer, since I can add that
-	     * many characters into the buffer after a backslash has occured.
+	     * many characters into the buffer after a backslash has occurred.
 	     */
 
 	    if (ptr + len > ptr_max) {
@@ -1532,7 +1532,7 @@ XrmPutStringResource(
     if (!*pdb) *pdb = NewDatabase();
     XrmStringToBindingQuarkList(specifier, bindings, quarks);
     value.addr = (XPointer) str;
-    value.size = strlen(str)+1;
+    value.size = (unsigned) strlen(str)+1;
     _XLockMutex(&(*pdb)->linfo);
     PutEntry(*pdb, bindings, quarks, XrmQString, &value);
     _XUnlockMutex(&(*pdb)->linfo);
@@ -1652,11 +1652,11 @@ GetIncludeFile(
 	len = str - base + 1;
 	if (len + fnamelen >= BUFSIZ)
 	    return;
-	strncpy(realfname, base, len);
-	strncpy(realfname + len, fname, fnamelen);
+	strncpy(realfname, base, (size_t) len);
+	strncpy(realfname + len, fname, (size_t) fnamelen);
 	realfname[len + fnamelen] = '\0';
     } else {
-	strncpy(realfname, fname, fnamelen);
+	strncpy(realfname, fname, (size_t) fnamelen);
 	realfname[fnamelen] = '\0';
     }
     if (!(str = ReadInFile(realfname)))
